//
//  GaugeView.m
//  Gyroscope
//
//  Created by Phidgets on 2015-07-24.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "GyroGaugeView.h"
static inline CGRect
convertToCGRect(NSRect rect)
{
    return CGRectMake(rect.origin.x, rect.origin.y,
                      rect.size.width, rect.size.height);
}
@implementation GyroGaugeView

- (id)initWithFrame:(NSRect)frameRect
{
    if(self = [super initWithFrame:frameRect])
    {
        img = [NSImage imageNamed:@"needle"];
    }
    
    return self;
}

-(void)setAngle:(double)deg{
    angle = deg * -M_PI/180;
    [self setNeedsDisplay:YES];
}

- (NSImage*)rotateImage:(NSImage*)orig byDegrees:(float)deg{
    NSImage *rotated = [[NSImage alloc] initWithSize:[orig size]];
    [rotated lockFocus];
    
    //Need to translate the image so it's centered at the origin before rotating
    NSAffineTransform *transform = [NSAffineTransform transform];
    [transform translateXBy:[orig size].width/2.0 yBy:[orig size].height/2.0];
    [transform rotateByRadians:deg];
    [transform translateXBy:-[orig size].width/2.0 yBy:-[orig size].height/2.0];
    [transform concat];
    
    [orig drawAtPoint:NSZeroPoint fromRect:NSZeroRect operation:NSCompositingOperationCopy fraction:1.0];
    [rotated unlockFocus];
    return rotated;
}

- (void)drawRect:(NSRect)re {
    [super drawRect:re];
    
    int i;
    float w, h, r;
    CGRect rect;
    CGContextRef context;
    
    rect = convertToCGRect(re);
    context = [[NSGraphicsContext currentContext] graphicsPort];
    
    w = CGRectGetWidth(rect);
    h = CGRectGetHeight(rect);
    r = (w < h) ? w/2.5 : h/2.5;

    //Draw the image
    [[NSGraphicsContext currentContext]
     setImageInterpolation: NSImageInterpolationHigh];
    
    NSSize viewSize  = [self bounds].size;
    double newWidth = (int)([img size].width * (h / [img size].height) * 0.8);
    double newHeight = (int)(h * 0.8 );
    NSSize imageSize = { newWidth, newHeight };
    
    NSPoint viewCenter;
    viewCenter.x = viewSize.width  * 0.50;
    viewCenter.y = viewSize.height * 0.50;
    
    NSPoint imageOrigin = viewCenter;
    imageOrigin.x -= imageSize.width  * 0.50;
    imageOrigin.y -= imageSize.height * 0.50;
    
    NSRect destRect;
    destRect.origin = imageOrigin;
    destRect.size = imageSize;
    
    NSImage *rotatedImage = [self rotateImage:[img copy] byDegrees:angle];
    [rotatedImage drawInRect:destRect fromRect:NSZeroRect
                   operation: NSCompositingOperationSourceOver
                    fraction: 1.0];
    
    /* Draw a cricle */
    CGContextBeginPath(context);
    CGContextAddArc(context, CGRectGetMidX(rect), CGRectGetMidY(rect), r, 0, 2*M_PI, false);
    CGContextSetRGBStrokeColor(context, 0,0,0, 1);
    CGContextStrokePath(context);
    
    //Ticks around the compass circle
    CGContextSetRGBStrokeColor(context, 0, 0, 0, 1);
    for (i = 0; i < 360; i+=5)
    {
        int tickSize = 8;
        CGContextSetLineWidth(context, 1);
        if (i == 0 || i == 45 || i == 90 || i == 135 || i == 180 || i == 225 || i == 270 || i == 315)
        {
            CGContextSetLineWidth(context, 2);
            tickSize = 16;
        }
        
        CGContextMoveToPoint(context, r * cos(i * (M_PI / 180.0)) + CGRectGetMidX(rect),
                             -r  * sin(i * (M_PI / 180.0)) + CGRectGetMidY(rect));
        CGContextAddLineToPoint(context, (r - tickSize) * cos(i * (M_PI / 180.0)) + CGRectGetMidX(rect),
                                -(r - tickSize) * sin(i * (M_PI / 180.0)) + CGRectGetMidY(rect));
        CGContextStrokePath(context);
    }
    //Draw labels
    NSDictionary *textAttributes = @{NSFontAttributeName: [NSFont systemFontOfSize:h/20]};
    [@"90" drawAtPoint:NSMakePoint(CGRectGetMidX(rect)-5, CGRectGetMidY(rect)+r+5) withAttributes:textAttributes];
    [@"-90" drawAtPoint:NSMakePoint(CGRectGetMidX(rect)-10, CGRectGetMidY(rect)-r-20) withAttributes:textAttributes];
    [@"180" drawAtPoint:NSMakePoint(CGRectGetMidY(rect)+r+40, CGRectGetMidY(rect)-7) withAttributes:textAttributes];
    [@"0" drawAtPoint:NSMakePoint(CGRectGetMidY(rect)-r+7, CGRectGetMidY(rect)-7) withAttributes:textAttributes];
}

@end
