//
//  GyroscopeController.m
//  Gyroscope
//
//  Created by Phidgets on 2015-07-24.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "GyroscopeController.h"

@implementation GyroscopeController

- (void)windowWillClose:(NSNotification*)aNotification {
    if(ch){
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetGyroscope_delete(&ch);
        ch = NULL;
    }
}


-(void)awakeFromNib{
    PhidgetReturnCode result;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    result = PhidgetGyroscope_create(&ch);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = PhidgetGyroscope_setOnAngularRateUpdateHandler(ch, gotAngularRateUpdateData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}



#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotDetach(PhidgetHandle phid, void *context){
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error){
        [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}
void gotAngularRateUpdateData(PhidgetGyroscopeHandle phid, void *context, const double angularRate[3], double timestamp){
    [(__bridge id)context performSelectorOnMainThread:@selector(onAngularRateDataHandler:)
                                           withObject:[NSArray arrayWithObjects: [NSNumber numberWithDouble:angularRate[0]],
                                                       [NSNumber numberWithDouble:angularRate[1]],[NSNumber numberWithDouble:angularRate[2]],[NSNumber numberWithDouble:timestamp],nil]
                                        waitUntilDone:NO];
}

#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    uint32 minDataInterval,maxDataInterval,dataInterval;
    
    //Get information from channel which will allow us to configure the GUI properly
    PhidgetGyroscope_getMaxDataInterval(ch, &maxDataInterval);
    PhidgetGyroscope_getMinDataInterval(ch, &minDataInterval);
    PhidgetGyroscope_getDataInterval(ch, &dataInterval);
    
    //Adjust GUI based on information from channel
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [settingsBox setHidden:NO];
    [dataBox setHidden:NO];
    [angularPositionsBox setHidden:NO];
    

    //Configure the data interval slider with information read from channel
    [minDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",minDataInterval]];
    [maxDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",maxDataInterval]];
    [dataIntervalSlider setMinValue:(double)minDataInterval];
    [dataIntervalSlider setMaxValue:(double)maxDataInterval];
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", dataInterval]];
    [dataIntervalSlider setIntValue:dataInterval];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow maxSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Reset
    for(int i = 0; i < 3; i++){
        gyroHeading[i] = 0;
        lastMsCountGood[i] = 0;
        lastMsCount[i] = 0.0;
    }
    //Taking items out of view
    [settingsBox setHidden:YES];
    [dataBox setHidden:YES];
    [angularPositionsBox setHidden:YES];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]
    	attributes:@{NSForegroundColorAttributeName: NSColor.controlTextColor}];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)outputLastError {
	const char *errorString;
	char *errorDetailString = NULL;
    size_t errorDetailStringLen = 0;
    PhidgetReturnCode lastErr;
	
    if (!Phidget_getLastError(&lastErr, &errorString, NULL, &errorDetailStringLen)) {
		errorDetailString = malloc(errorDetailStringLen);
		Phidget_getLastError(&lastErr, &errorString, errorDetailString, &errorDetailStringLen);
    	[self outputError:errorDetailString];
    	free(errorDetailString);
    }
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

- (void)onAngularRateDataHandler:(NSArray *)gyroData{
    double x = [[gyroData objectAtIndex:0]doubleValue];
    double y = [[gyroData objectAtIndex:1]doubleValue];
    double z = [[gyroData objectAtIndex:2]doubleValue];
    [xAxisText setStringValue:[NSString stringWithFormat:@"%f º/s",x]];
    [yAxisText setStringValue:[NSString stringWithFormat:@"%f º/s",y]];
    [zAxisText setStringValue:[NSString stringWithFormat:@"%f º/s",z]];
    [timeStampText setStringValue:[NSString stringWithFormat:@"%.0f ms",[[gyroData objectAtIndex:3]doubleValue]]];
    
    //Do not animate if saturation occurs to avoid problems 
    if(x == PUNK_DBL || y == PUNK_DBL || z == PUNK_DBL)
        return;
    
    for(int i = 0; i < 3; i++)
        [self calculateGyroHeading:gyroData :i];
    
    [xGauge setAngle:gyroHeading[0]];
    [yGauge setAngle:gyroHeading[1]];
    [zGauge setAngle:gyroHeading[2]];

}

int lastMsCountGood[3] = {0,0,0};
double gyroHeading[3] = {0,0,0};
double lastMsCount[3] = {0,0,0};
-(void)calculateGyroHeading:(NSArray *)gyroData
                           :(int)index{
    double gy = 0;
    gy = [[gyroData objectAtIndex:index]doubleValue];
    if(lastMsCountGood[index]){
        //calculate heading
        double timechange = [[gyroData objectAtIndex:3]doubleValue] - lastMsCount[index]; //in ms
        double timeChangeSeconds = (double)timechange / 1000.0;
        
        gyroHeading[index] += timeChangeSeconds * gy;
        
        //ensure proper wrap around
        if(gyroHeading[index] > 180)
            gyroHeading[index] -= 360;
        else if (gyroHeading[index] < -180)
            gyroHeading[index] += 360;
    }

    lastMsCount[index] = [[gyroData objectAtIndex:3]doubleValue];
    lastMsCountGood[index] = 1;
}

#pragma mark GUI controls
- (IBAction)setDataInterval:(id)sender{
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetGyroscope_setDataInterval(ch, [dataIntervalSlider intValue]);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
    }
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", [dataIntervalSlider intValue]]];
}
- (IBAction)zeroGyro:(id)sender{
    PhidgetReturnCode result;
    result = PhidgetGyroscope_zero(ch);
    if(result != EPHIDGET_OK){
        [self outputLastError];
        return;
    }
    for(int i = 0; i < 3; i++)
        gyroHeading[i] = 0;
}


@end
